package br.com.ases.controller;

import static br.com.checker.emag.core.Checker.behavior;
import static br.com.checker.emag.core.Checker.content;
import static br.com.checker.emag.core.Checker.form;
import static br.com.checker.emag.core.Checker.from;
import static br.com.checker.emag.core.Checker.marking;
import static br.com.checker.emag.core.Checker.multimedia;
import static br.com.checker.emag.core.Checker.presentation;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletContext;
import javax.ws.rs.QueryParam;

import org.apache.poi.hssf.record.PageBreakRecord.Break;

import net.sf.jasperreports.engine.JRException;
import br.com.ases.business.AvaliacaoBusiness;
import br.com.ases.controller.EseloController.Nota;
import br.com.ases.domain.DetalheAvaliacao;
import br.com.ases.domain.DetalheAvaliacao.Criterio;
import br.com.ases.domain.OccurrenceKey;
import br.com.ases.domain.ResumoAvaliacao;
import br.com.ases.infra.CriterioProperties;
import br.com.ases.infra.EseloProperties;
import br.com.ases.infra.UtilitiesProperties;
import br.com.ases.infra.WebChecker;
import br.com.ases.model.utilities.DateUtil;
import br.com.ases.model.utilities.ManagerReport;
import br.com.ases.model.utilities.Validate;
import br.com.ases.ws.model.AvaliacaoAutomatica;
import br.com.ases.ws.model.CodigoFonteCriterio;
import br.com.ases.ws.model.CriterioWs;
import br.com.ases.ws.model.ErroWs;
import br.com.ases.ws.model.PaginaAvaliada;
import br.com.ases.ws.model.RecomendacaoAvaliacao;
import br.com.ases.ws.model.ResumoAvaliacaoWs;
import br.com.ases.ws.model.Secao;
import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.interceptor.download.FileDownload;
import br.com.caelum.vraptor.interceptor.multipart.UploadedFile;
import br.com.caelum.vraptor.ioc.spring.VRaptorRequestHolder;
import br.com.caelum.vraptor.validator.Message;
import br.com.caelum.vraptor.view.Results;
import br.com.checker.emag.Occurrence;
import br.com.checker.emag.OccurrenceClassification;
import br.com.checker.emag.SummarizedOccurrence;
import br.com.checker.emag.core.Checker;
import br.com.checker.emag.core.ContentEvaluation;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;


@Resource
public class AvaliacaoController {
  EseloProperties eseloProperties = null;

  private String tituloPagina;
  // Altera a mensagem caso n�o esteja usando o ESELO (Nota e Resumo da
  // Avalia��o de Acessibilidade)
  private String mensagem_avaliacao = null;

  // Esconde a nota caso n�o esteja usando o ESELO
  private String sem_nota = null;
  private String tituloSite;
  private String ancoraCampoVazio;
  private String dataHoraAvaliacao;
  private String webaxscore;
  private Result result;
  private List<String> avisosFerramentasHtmlCssW3c;
  private Validator validator;
  private AvaliacaoBusiness avaliacaoBusiness;
  private Map<OccurrenceClassification, List<SummarizedOccurrence>> ocorrencias =
      new HashMap<OccurrenceClassification, List<SummarizedOccurrence>>();
  private ServletContext application;
  private DetalheAvaliacao detalheAvaliacao;
  private UtilitiesProperties utilitiesProperties;
  // private StringBuffer xml;
  private CriterioProperties criterioProperties;
  private String chaveIdentificaAvaliacao;
  private String novaChave;

  // private static int PRETTY_PRINT_INDENT_FACTOR = 4;

  public AvaliacaoController(Result result, Validator validator,
      AvaliacaoBusiness avaliacaoBusiness, ServletContext application,
      DetalheAvaliacao detalheAvaliacao) {
    this.result = result;
    this.validator = validator;
    this.avaliacaoBusiness = avaliacaoBusiness;
    this.application = application;
    this.detalheAvaliacao = detalheAvaliacao;
    this.avaliacaoBusiness.initEseloProperties(application);
    this.initEseloProperties(application);
    this.criterioProperties = new CriterioProperties(application);
    this.utilitiesProperties = new UtilitiesProperties(application);
    this.chaveIdentificaAvaliacao =
        DateUtil.dataHoraMinutoAtual().replace("/", "").replace(":", "").replace(" ", "").replace(".", "");
    result.include("chaveIdentificaAvaliacao", chaveIdentificaAvaliacao);
    
    //VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("chaveIdentificaAvaliacao", chaveIdentificaAvaliacao);

  }


  private void DefinirCorWebaxscore(String valorNota) {

    // Define a cor do webaxscore na p�gina "Avaliar" de acordo a pontua��o

    Double notaAvaliacao = Double.parseDouble(valorNota.replaceFirst(",", "."));

    if (notaAvaliacao >= 95) {
        webaxscore = "bom";      
      } else if (notaAvaliacao >= 85 & notaAvaliacao < 95) {
          webaxscore = "razoavel";
      } else if (notaAvaliacao >= 70 & notaAvaliacao < 85) {
          webaxscore = "ruim";
      }else {
        webaxscore = "pessimo";
      }
    

  }


  @Path("/avaliar-arquivo")
  public void avaliarArquivo(UploadedFile file, boolean mark, boolean content,
      boolean presentation, boolean multimedia, boolean form, boolean behavior, int tiprel)
      throws IOException {

    Validate validate = new Validate(this.validator);
    if (validate.uploadForm(file)) {

      BufferedReader reader = new BufferedReader(new InputStreamReader(file.getFile()));
      String html = "";
      String linha = "";
      while ((linha = reader.readLine()) != null)
        // html += "\n"+linha;
        html += linha + "\n";

      if (validate.conteudoUploadForm(html)) {

        /*
         * if(tiprel != 5) this.result.redirectTo(AvaliacaoController.class
         * ).relatorioAvaliacao(html, mark, content, presentation, multimedia, form, behavior,
         * tiprel, false);
         */

        Checker checker = from(html);

        if (mark)
          checker.with(marking());
        if (content)
          checker.with(content());
        if (presentation)
          checker.with(presentation());
        if (multimedia)
          checker.with(multimedia());
        if (form)
          checker.with(form());
        if (behavior)
          checker.with(behavior());

        html = html.replaceAll("<", "&lt;");
        html = html.replaceAll(">", "&gt;");
        html = html.replaceAll(" ", "&nbsp");

        result.include("contentLenght", String.valueOf(html.getBytes("UTF-8").length));
        result.include("html", html);
        result.include("contentLenght", String.valueOf(html.getBytes("UTF-8").length));
        result.include("html", html);

        this.tituloSite = "";

        ContentEvaluation conteudo = new ContentEvaluation(checker.getDocument());
        this.tituloSite = conteudo.retornarTituloSiteAvaliado();

        result.include("titulosite", tituloSite);
        result.include("titulosite", tituloSite);
        Nota nota = null;

        if (!this.sem_nota.equalsIgnoreCase("sem_nota")) {
          List<Entry<OccurrenceKey, List<Occurrence>>> mapaListaOcorrencias =
              avaliacaoBusiness.retornarCriteriosTeste(checker.check());
          // nota = avaliacaoBusiness.obterNota(checker.checkSumarized(), file.getFileName(),
          // mapaListaOcorrencias);
          nota =
              avaliacaoBusiness.obterNotaEselo(checker.getDocument(), file.getFileName(),
                  mapaListaOcorrencias, false);
          // Altera a cor de webaxscore de acordo a pontuacao

          if (nota != null) {
            DefinirCorWebaxscore(nota.getValor());

            VRaptorRequestHolder.currentRequest().getServletContext()
                .setAttribute("webaxscore", webaxscore);

            novaChave = "webaxscore" + chaveIdentificaAvaliacao;
            VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, webaxscore);
          }

        }

        result.include("nota", nota);
        result.include("nota", nota);
        this.sumarizarResultasNoResponse(checker.checkSumarized(), result);
        this.detalheAvaliacao.inicializar(avaliacaoBusiness.retornarCriterios(checker.check()));



        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("resultadoAvaliacao", checker.checkSumarized());

        novaChave = "resultadoAvaliacao" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute(novaChave, checker.checkSumarized());

        VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("urlAvaliada", "");

        novaChave = "urlAvaliada" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, "");

        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("contentLenght", String.valueOf(html.getBytes("UTF-8").length));

        novaChave = "contentLenght" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute(novaChave, String.valueOf(html.getBytes("UTF-8").length));


        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("notaAvaliacao", nota);

        novaChave = "notaAvaliacao" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, nota);



        // Seta o valor do t�tulo no template
        tituloPagina = "Resumo de avaliação por upload de arquivo - ASES";
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("tituloPagina", tituloPagina);

        novaChave = "tituloPagina" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute(novaChave, tituloPagina);

        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("titulosite", tituloSite);

        novaChave = "titulosite" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute(novaChave, tituloSite);

        this.dataHoraAvaliacao = (String) DateUtil.dataHoraAtual();
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("data", dataHoraAvaliacao);

        novaChave = "data" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute(novaChave, dataHoraAvaliacao);

        result.of(this).avaliar(null, mark, content, presentation, multimedia, form, behavior,
            tiprel);

      } else {

        // Atribui �ncora para as mensagens de erros dos campos vazios
        ancoraCampoVazio = "up_file";
        VRaptorRequestHolder.currentRequest().getServletContext()
            .setAttribute("ancoraCampoVazio", ancoraCampoVazio);

        novaChave = "ancoraCampoVazio" + chaveIdentificaAvaliacao;
        VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, ancoraCampoVazio);

        this.validator = validate.getMessage();
        this.validator.onErrorUsePageOf(IndexController.class).index();
      }

    } else {
      // Atribui �ncora para as mensagens de erros dos campos vazios
      ancoraCampoVazio = "up_file";
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("ancoraCampoVazio", ancoraCampoVazio);

      novaChave = "ancoraCampoVazio" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, ancoraCampoVazio);

      this.validator = validate.getMessage();
      this.validator.onErrorUsePageOf(IndexController.class).index();
    }
  }

  @Post("/avaliar")
  public void avaliar(String url, boolean mark, boolean content, boolean presentation,
      boolean multimedia, boolean form, boolean behavior, int tiporel) {

    Validate validate = new Validate(this.validator);
    if (!url.startsWith("http"))
    {
      url = "http://" + url;
    }
    
    if (validate.url(url)) {

      /*
       * if(tiporel != 5) this.result.redirectTo(AvaliacaoController.class) .relatorioAvaliacao(url,
       * mark, content, presentation, multimedia, form, behavior, tiporel, true);
       */

     
      WebChecker pagina = WebChecker.from(url).withGetRequest().execute();
      
      

      int[] errorsWarningsCss = avaliacaoBusiness.getErrorCount(true, url);
      int[] errorsWarningsHtml = avaliacaoBusiness.getErrorCount(false, url);

 
      
      Checker checker = from(pagina.getContent(), url);

      if (mark)
        checker.with(marking());
      if (content)
        checker.with(content());
      if (presentation)
        checker.with(presentation());
      if (multimedia)
        checker.with(multimedia());
      if (form)
        checker.with(form());
      if (behavior)
        checker.with(behavior());

      Pattern pp = Pattern.compile("(http://www.)?([a-z]*)(.)?");
      Matcher mm = pp.matcher(url.toLowerCase().replace("https", "http"));

      if (mm.find())

        this.tituloSite = "";

      ContentEvaluation conteudo = new ContentEvaluation(checker.getDocument());

      this.tituloSite = conteudo.retornarTituloSiteAvaliado();


      result.include("titulosite", tituloSite);

      result.include("contentLenght", pagina.getContentLength());

      result.include("url", url);

      result.include("html", pagina.getParsedContent());

      Nota nota = null;

      if (!this.sem_nota.equalsIgnoreCase("sem_nota")) {

        List<Entry<OccurrenceKey, List<Occurrence>>> mapaListaOcorrencias =
            avaliacaoBusiness.retornarCriteriosTeste(checker.check());
        // Altera a cor de webaxscore de acordo a pontuacao
        // nota = avaliacaoBusiness.obterNota(checker.checkSumarized(), url, mapaListaOcorrencias);

        nota = avaliacaoBusiness.obterNotaEselo(checker.getDocument(), url, mapaListaOcorrencias, false);

        if (nota != null) {
          DefinirCorWebaxscore(nota.getValor());

          VRaptorRequestHolder.currentRequest().getServletContext()
              .setAttribute("webaxscore", webaxscore);

          novaChave = "webaxscore" + chaveIdentificaAvaliacao;
          VRaptorRequestHolder.currentRequest().getServletContext()
              .setAttribute(novaChave, webaxscore);
        }

      }

      result.include("nota", nota);
      this.sumarizarResultasNoResponse(checker.checkSumarized(), result);

      this.detalheAvaliacao.inicializar(avaliacaoBusiness.retornarCriterios(checker.check()),
          errorsWarningsCss, errorsWarningsHtml);


      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("resultadoAvaliacao", checker.checkSumarized());

      novaChave = "resultadoAvaliacao" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, checker.checkSumarized());

      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("urlAvaliada", url);

      novaChave = "urlAvaliada" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, url);

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("contentLenght", pagina.getContentLength());

      novaChave = "contentLenght" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, pagina.getContentLength());

      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("notaAvaliacao", nota);
      novaChave = "notaAvaliacao" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, nota);

      // Seta o valor do t�tulo no template
      tituloPagina = "Resumo de avaliação por URI - ASES";
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("tituloPagina", tituloPagina);

      novaChave = "tituloPagina" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, tituloPagina);

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("titulosite", tituloSite);

      novaChave = "titulosite" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, tituloSite);

      this.dataHoraAvaliacao = (String) DateUtil.dataHoraAtual();

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("data", dataHoraAvaliacao);

      novaChave = "data" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, dataHoraAvaliacao);

      avisosFerramentasHtmlCssW3c = new ArrayList<String>();

      validarFerramenta_cssnoAr(errorsWarningsCss);
      validarFerramenta_htmlnoAr(errorsWarningsHtml);

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("avisosFerramentasHtmlCssW3c", avisosFerramentasHtmlCssW3c);

      novaChave = "avisosFerramentasHtmlCssW3c" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, avisosFerramentasHtmlCssW3c);

    } else {
      // Atribui �ncora para as mensagens de erros dos campos vazios
      ancoraCampoVazio = "url";
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("ancoraCampoVazio", ancoraCampoVazio);

      novaChave = "ancoraCampoVazio" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, ancoraCampoVazio);

      this.validator = validate.getMessage();
      this.validator.onErrorUsePageOf(IndexController.class).index();
    }
  }

  // Verifica se o site de avalia��o de sintaxe css encontra-se no ar
  private void validarFerramenta_cssnoAr(int[] erros_avisos) {
    if (erros_avisos[0] == -10 && erros_avisos[1] == -10) {

      avisosFerramentasHtmlCssW3c
          .add("O avaliador de sintaxe css do w3c encontra-se indispon&iacute;vel, favor tentar mais tarde.");
    }

  }

  // Verifica se o site de avalia��o de sintaxe html encontra-se no ar
  private void validarFerramenta_htmlnoAr(int[] erros_avisos) {
    if (erros_avisos[0] == -10 && erros_avisos[1] == -10) {
      avisosFerramentasHtmlCssW3c
          .add("O avaliador de sintaxe html do w3c encontra-se indispon&iacute;vel, favor tentar mais tarde.");
    }
  }

  @Get("/relatorioavaliacao")
  @Post("/relatorioavaliacao")
  public FileDownload relatorioAvaliacao(int tiporel, String chaveAvaliacao) {

    novaChave = "resultadoAvaliacao" + chaveAvaliacao;
    List<SummarizedOccurrence> checkerList =
        (List<SummarizedOccurrence>) VRaptorRequestHolder.currentRequest().getServletContext()
            .getAttribute(novaChave);

  

    novaChave = "contentLenght" + chaveAvaliacao;
    String contentLenght =
        (String) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave);
  
    this.sumarizarResultasNoResponse(checkerList, result);

    // ================================================ GERAR RELAT�RIO
    // =============================================//

    /* Cria um Map de par�metros */
    HashMap<String, Object> map = new HashMap<String, Object>();

    /* Obtem a nota */
    // AvaliacaoBusinessImpl avaliacaoBusiness = new
    // AvaliacaoBusinessImpl();
    Nota nota = null;

    if (!this.sem_nota.equalsIgnoreCase("sem_nota")) {
      novaChave = "notaAvaliacao" + chaveAvaliacao;
      nota =
          (Nota) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave);

      map.put("pPercentualAses", nota.getValor());
    } else {
      map.put("pPercentualAses", "0");
    }

    novaChave = "urlAvaliada" + chaveAvaliacao;
    String urlAvaliada = (String) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave);
    map.put("pPagina", urlAvaliada != "" ? urlAvaliada : "Código Fonte ou Arquivo");

    if (urlAvaliada != "") {
      Pattern pp = Pattern.compile("(http://www.)?([a-z]*)(.)?");
      Matcher mm = pp.matcher(urlAvaliada.toLowerCase());

      if (mm.find())
        novaChave = "titulosite" + chaveAvaliacao;
      map.put("pTitulo",
          VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave));

    } else
      novaChave = "titulosite" + chaveAvaliacao;
    map.put("pTitulo",
        VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave));



    map.put("pTamanho", contentLenght + " Bytes");
    novaChave = "data" + chaveAvaliacao;
    dataHoraAvaliacao =
        (String) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave);
    map.put("pDataHoraAvaliacao", dataHoraAvaliacao);

    // Obtem Resumo da Avaliação
    List<ResumoAvaliacao> resumoErrosAvisos = obterResumoAvaliacao();

    int totalErros = 0;
    int totalAvisos = 0;

    for (ResumoAvaliacao resumo : resumoErrosAvisos) {
      map.put(resumo.getTipo() + "_A", resumo.getQuantidadeAvisos());
      map.put(resumo.getTipo() + "_E", resumo.getQuantidadeErros());
      totalErros += resumo.getQuantidadeErros();
      totalAvisos += resumo.getQuantidadeAvisos();
    }

    map.put("TOTAL_E", totalErros);
    map.put("TOTAL_A", totalAvisos);

    ManagerReport managerReport =
        new ManagerReport(this.application.getRealPath("")
            + "/WEB-INF/templates-relatorios/relatorio-avaliacao.jrxml");
   
    
    String path = null;

    try {
      path = managerReport.gerarRelatorio(checkerList, map, tiporel, "RelatorioAvaliacao");
    } catch (JRException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
    File file = new File(path);

    return new FileDownload(file, managerReport.getContentType(), managerReport.getFileName());

    // =========================================== FIM GERAR RELAT�RIO
    // =============================================//

  }

  @Get("/avaliar-codigo")
  @Post("/avaliar-codigo")
  public void avaliarPlainText(String html, boolean mark, boolean content, boolean presentation,
      boolean multimedia, boolean form, boolean behavior, int tiporel) throws IOException {

    Validate validate = new Validate(this.validator);
    if (validate.condigoFonte(html)) {


      Checker checker = from(html);

      if (mark)
        checker.with(marking());
      if (content)
        checker.with(content());
      if (presentation)
        checker.with(presentation());
      if (multimedia)
        checker.with(multimedia());
      if (form)
        checker.with(form());
      if (behavior)
        checker.with(behavior());

      html = html.replaceAll("<", "&lt;");
      html = html.replaceAll(">", "&gt;");
      html = html.replaceAll(" ", "&nbsp");

      SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy H:mm:ss");

      result.include("contentLenght", String.valueOf(html.getBytes("UTF-8").length));
      result.include("html", html);
      this.tituloSite = "";

      ContentEvaluation conteudo = new ContentEvaluation(checker.getDocument());

      this.tituloSite = conteudo.retornarTituloSiteAvaliado();

      result.include("titulosite", tituloSite);

      Nota nota = null;

      if (!this.sem_nota.equalsIgnoreCase("sem_nota")) {

        List<Entry<OccurrenceKey, List<Occurrence>>> mapaListaOcorrencias =
            avaliacaoBusiness.retornarCriteriosTeste(checker.check());
        nota =
            avaliacaoBusiness.obterNotaEselo(checker.getDocument(),
                tituloSite + " - " + sdf.format(new Date()), mapaListaOcorrencias, false);
        // Altera a cor de webaxscore de acordo a pontuacao

        if (nota != null) {
          DefinirCorWebaxscore(nota.getValor());

          VRaptorRequestHolder.currentRequest().getServletContext()
              .setAttribute("webaxscore", webaxscore);
        }


      }

      result.include("nota", nota);
      this.sumarizarResultasNoResponse(checker.checkSumarized(), result);
      this.detalheAvaliacao.inicializar(avaliacaoBusiness.retornarCriterios(checker.check()));

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("resultadoAvaliacao", checker.checkSumarized());

      novaChave = "resultadoAvaliacao" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, checker.checkSumarized());

      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("urlAvaliada", "");

      novaChave = "urlAvaliada" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, "");

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("contentLenght", String.valueOf(html.getBytes("UTF-8").length));

      novaChave = "notaAvaliacao" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("contentLenght", String.valueOf(html.getBytes("UTF-8").length));
      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, nota);

      // Seta o valor do t�tulo no template
      tituloPagina = "Resumo de avaliação por código fonte - ASES";
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("tituloPagina", tituloPagina);

      novaChave = "tituloPagina" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, tituloPagina);

      result.of(this).avaliar(null, mark, content, presentation, multimedia, form, behavior,
          tiporel);

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("titulosite", tituloSite);

      novaChave = "titulosite" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, tituloSite);

      this.dataHoraAvaliacao = (String) DateUtil.dataHoraAtual();


      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("data", dataHoraAvaliacao);

      novaChave = "data" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, dataHoraAvaliacao);

    } else {
      // Atribui �ncora para as mensagens de erros dos campos vazios
      ancoraCampoVazio = "input";

      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute("ancoraCampoVazio", ancoraCampoVazio);

      novaChave = "ancoraCampoVazio" + chaveIdentificaAvaliacao;
      VRaptorRequestHolder.currentRequest().getServletContext()
          .setAttribute(novaChave, ancoraCampoVazio);

      this.validator = validate.getMessage();
      this.validator.onErrorUsePageOf(IndexController.class).index();
    }
  }

  private void sumarizarResultasNoResponse(List<SummarizedOccurrence> resultadoAvaliacao,
      Result result) {

    for (SummarizedOccurrence occurrence : resultadoAvaliacao) {
      if (ocorrencias.get(occurrence.getType()) == null)
        ocorrencias.put(occurrence.getType(), new ArrayList<SummarizedOccurrence>());

      ocorrencias.get(occurrence.getType()).add(occurrence);
    }

    for (Entry<OccurrenceClassification, List<SummarizedOccurrence>> entry : ocorrencias.entrySet()) {
      List<SummarizedOccurrence> ocorrenciasComputadas = entry.getValue();
      Collections.sort(ocorrenciasComputadas);

      result.include("LISTA_" + entry.getKey().toString(), ocorrenciasComputadas);
    }

    List<ResumoAvaliacao> resumoErrosAvisos = obterResumoAvaliacao();
    int totalErros = 0;
    int totalAvisos = 0;

    for (ResumoAvaliacao resumo : resumoErrosAvisos) {
      totalErros += resumo.getQuantidadeErros();
      totalAvisos += resumo.getQuantidadeAvisos();
    }

    result.include("totalErros", totalErros);
    result.include("totalAvisos", totalAvisos);
    result.include("listaResumo", resumoErrosAvisos);
    result.include("data", DateUtil.dataHoraAtual());

  }

  private List<ResumoAvaliacao> obterResumoAvaliacao() {
    List<ResumoAvaliacao> resultado = new ArrayList<ResumoAvaliacao>();

    for (OccurrenceClassification classificacao : OccurrenceClassification.values()) {

      int erros = 0;
      int avisos = 0;

      List<SummarizedOccurrence> listaOcorrencias = ocorrencias.get(classificacao);

      if (listaOcorrencias != null) {
        for (SummarizedOccurrence ocorrencia : ocorrencias.get(classificacao)) {

          if (ocorrencia.isError())
            for (int line : ocorrencia.getLines())
              erros++;
          else
            for (int line : ocorrencia.getLines())
              avisos++;

        }
      }

      resultado.add(new ResumoAvaliacao(classificacao, erros, avisos));
    }

    return resultado;
  }


  // Gibran
  public void obterDetalhes(OccurrenceKey rn, boolean type) {


    List<Occurrence> listOcorrencias = this.detalheAvaliacao.get(rn, type).getOcorrencias();



    Collections.sort(listOcorrencias, new Comparator<Occurrence>() {
      public int compare(Occurrence occurrence1, Occurrence occurrence2) {
        return occurrence1.getLine().compareTo(occurrence2.getLine());
      }
    });


    result.include("detalhe", this.detalheAvaliacao.get(rn, type));
  }

  @Path("/detalhes-avaliacao/{rn}/{type}/{chaveAvaliacao}")
  public void detalhesAvaliacao(OccurrenceKey rn, boolean type, String chaveAvaliacao) {
    String url = "";
    tituloPagina = "Detalhes da avalição - ASES";

    result.include("chaveIdentificaAvaliacao", chaveAvaliacao);

    novaChave = chaveIdentificaAvaliacao + "tituloPagina";
    VRaptorRequestHolder.currentRequest().getServletContext().setAttribute(novaChave, tituloPagina);

    List<Occurrence> listOcorrencias = this.detalheAvaliacao.get(rn, type).getOcorrencias();


    // Sorting
    Collections.sort(listOcorrencias, new Comparator<Occurrence>() {
      public int compare(Occurrence occurrence1, Occurrence occurrence2) {
        return occurrence1.getLine().compareTo(occurrence2.getLine());
      }
    });

    result.include("detalhe", this.detalheAvaliacao.get(rn, type));
    result.include("listOcorrencia", listOcorrencias);

    result.include("isError", type);
    if (type) {
      result.include("qtdOcorrenciasCss", this.detalheAvaliacao.getErrorsCss());
      result.include("qtdOcorrenciasHtml", this.detalheAvaliacao.getErrorsHtml());
    } else {
      result.include("qtdOcorrenciasCss", this.detalheAvaliacao.getWarningsCss());
      result.include("qtdOcorrenciasHtml", this.detalheAvaliacao.getWarningsHtml());
    }

    novaChave = "resultadoAvaliacao" + chaveAvaliacao;
    List<SummarizedOccurrence> ob =
        (List<SummarizedOccurrence>) VRaptorRequestHolder.currentRequest().getServletContext()
            .getAttribute(novaChave);
    String recomendacao = "";


    for (SummarizedOccurrence occurrence : ob) {
      recomendacao = occurrence.getMapDescription().get(rn.getCode());
      break;
    }

    result.include("recomendacao", recomendacao);
    result.include("rn", rn.getCode());
    result.include("aReq", this.utilitiesProperties.get("notExibCrit"));
    result.include("aReqIsCss", this.utilitiesProperties.get("reqCss"));
    result.include("aReqIsW3c", this.utilitiesProperties.get("notExibCritW3c"));
    novaChave = "urlAvaliada" + chaveAvaliacao;
    url =
        VRaptorRequestHolder.currentRequest().getServletContext().getAttribute(novaChave)
            .toString().replaceAll("(https://)|(http://)", "");
    result.include("url", url);

    try {

      result.include("urlFormatada", encodeURI(url));

    } catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }
    result.include("linkAvaliacaoCSSExterno",this.eseloProperties.getUrl("linkAvaliacaoCSSExterno"));
    result.include("linkAvaliacaoHtmlExterno",this.eseloProperties.getUrl("linkAvaliacaoHtmlExterno"));
  }

  private String encodeURI(String url) throws UnsupportedEncodingException {
    return URLEncoder.encode(url, "UTF-8").replace("+", "%20").replace("%21", "!")
        .replace("%27", "'").replace("%28", "(").replace("%29", ")").replace("%7E", "~");
  }

  @Post("/exportar-detalhes-avaliacao")
  public FileDownload exportarDetalhesAvaliacao(OccurrenceKey rn, int tiporel, boolean isError,
      String chaveAvaliacao) {

    /* Cria um Map de parametros */
    HashMap<String, Object> map = new HashMap<String, Object>();

    List<String> aReq = new ArrayList();

    String[] crit = this.utilitiesProperties.get("notExibCrit").split(" ");

    for (String c : crit)
      aReq.add(c.trim());

    List<Criterio> listCrit = this.detalheAvaliacao.get(rn, isError).getCriterios();

    List<Criterio> list = new ArrayList<DetalheAvaliacao.Criterio>();

    for (Criterio criterio : listCrit) {
      if (aReq.contains(rn.getCode() + "." + criterio.getId()))
        criterio.setLinhas(new ArrayList());

      if (criterio.getId() != null)
        list.add(criterio);
    }

    List<String> codigoFonte = new ArrayList();

    for (Occurrence occurrence : this.detalheAvaliacao.get(rn, isError).getOcorrencias()) {
      if (!aReq.contains(rn.getCode() + "." + occurrence.getCriterio()))
        codigoFonte.add(occurrence.getLine()
            + ": "
            + occurrence.getTag().replaceAll("&lt;", "<").replaceAll("&gt;", ">")
                .replaceAll("&nbsp", " ") + "\n\n");
    }
    map.put("codigoFonte", codigoFonte);

    novaChave = "resultadoAvaliacao" + chaveAvaliacao;
    List<SummarizedOccurrence> ob =
        (List<SummarizedOccurrence>) VRaptorRequestHolder.currentRequest().getServletContext()
            .getAttribute(novaChave);
    String recomendacao = "";

    for (SummarizedOccurrence occurrence : ob) {
      recomendacao = occurrence.getMapDescription().get(rn.getCode());
      break;
    }

    map.put("rnAvaliada", recomendacao);
    map.put("SUBREPORT_DIR", this.application.getRealPath("") + "/WEB-INF/templates-relatorios/");

    ManagerReport managerReport =
        new ManagerReport(this.application.getRealPath("")
           + "/WEB-INF/templates-relatorios/relatorio-detalhes-avaliacao.jrxml");

   
    
    String path = null;

    try {
      path =
          managerReport.gerarRelatorioDetalhesAvaliacao(list, map, tiporel,
              "DetalhesRelatorioAvaliacao");
    } catch (JRException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }

    File file = new File(path);

    return new FileDownload(file, managerReport.getContentType(), managerReport.getFileName());
  }

  public void initEseloProperties(ServletContext servletContext) {
    this.eseloProperties = new EseloProperties(servletContext);

    // Esconde a nota caso n�o esteja usando o ESELO
    sem_nota = this.eseloProperties.getSem_nota("sem_nota");
    VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("sem_nota", sem_nota);

    // Altera a mensagem caso n�o esteja usando o ESELO (Nota e Resumo da
    // Avalia��o de Acessibilidade)
    mensagem_avaliacao = this.eseloProperties.getMensagem_avaliacao("mensagem_avaliacao");
    VRaptorRequestHolder.currentRequest().getServletContext()
        .setAttribute("mensagem_avaliacao", mensagem_avaliacao);
  }

  @Get("/servico-avaliacao-acessibilidade-salvando")
  @Post("/servico-avaliacao-acessibilidade-salvando")
  public void servicoAvaliacaoAcessibilidadeSalvando(@QueryParam("url") String url) {

	//Boolean retornoNota = false;
	String retornoNota = "false";
    ErroWs erroWs = new ErroWs();
    this.dataHoraAvaliacao = (String) DateUtil.dataHoraAtual();


             try {

          Validate validate = new Validate(this.validator);

          if (!url.startsWith("http")) {
              url = "http://" + url;
            }
          
          if (validate.url(url)) {

           


            WebChecker pagina = WebChecker.from(url).withGetRequest().execute();

            int[] errorsWarningsCss = avaliacaoBusiness.getErrorCount(true, url);
            int[] errorsWarningsHtml = avaliacaoBusiness.getErrorCount(false, url);

            Checker checker = from(pagina.getContent(), url);

            checker.with(marking());
            checker.with(content());
            checker.with(presentation());
            checker.with(multimedia());
            checker.with(form());
            checker.with(behavior());

            Pattern pp = Pattern.compile("(http://www.)?([a-z]*)(.)?");
            Matcher mm = pp.matcher(url.toLowerCase().replace("https", "http"));

            if (mm.find()) {
              this.tituloSite = "";
            }

            ContentEvaluation conteudo = new ContentEvaluation(checker.getDocument());

            this.tituloSite = conteudo.retornarTituloSiteAvaliado();

            Nota nota = null;

            List<Entry<OccurrenceKey, List<Occurrence>>> mapaListaOcorrencias =
                avaliacaoBusiness.retornarCriteriosTeste(checker.check());

            nota =
                avaliacaoBusiness.obterNotaEselo(checker.getDocument(), url, mapaListaOcorrencias, true);
            if(nota != null)
            {
            	retornoNota = "true";
            }
            
          }
          else
          {
        	  retornoNota = "Url não responde";
          }

        } catch (Exception e) {
       
          e.printStackTrace();

          erroWs.setUrl(url);
          erroWs.setDataHora(this.dataHoraAvaliacao);

          List<String> mensagensErros = new ArrayList<String>();

          mensagensErros.add(e.getMessage());
          erroWs.setMensagem(mensagensErros);
          retornoNota = e.getMessage();

        }    
             finally
             {
            	 result.use(Results.json()).withoutRoot().from(retornoNota).serialize();  
             }
             
  }

  @Get("/servico-avaliacao-acessibilidade")
  @Post("/servico-avaliacao-acessibilidade")
  public void servicoAvaliacaoAcessibilidade(@QueryParam("url") String url, String tipoSaida,
      String codigoHash) {

    AvaliacaoAutomatica avaliacaoAutomatica = new AvaliacaoAutomatica();
    ErroWs erroWs = new ErroWs();
    this.dataHoraAvaliacao = (String) DateUtil.dataHoraAtual();
    WebChecker.PostParams postParams =
        WebChecker.from(this.eseloProperties.getUrl("urlAutenticacaoServico")).withPostRequest();
    postParams.addParam("codigoHash", codigoHash);

    Gson g = new GsonBuilder().create();

    Boolean usuarioAutorizado = g.fromJson(postParams.execute().getContent(), Boolean.class);


    if (url == null || tipoSaida == null || codigoHash == null ||(!tipoSaida.equalsIgnoreCase("xml")
        && !tipoSaida.equalsIgnoreCase("json"))) {
      
      erroWs.setDataHora(this.dataHoraAvaliacao);

      List<String> mensagensErros = new ArrayList<String>();

      if (url == null)
      {
        erroWs.setUrl("");
        mensagensErros.add("Favor, verificar o parâmetro 'url'!");
      }
      else
      {
        erroWs.setUrl(url);
      }
      if (tipoSaida == null||(!tipoSaida.equalsIgnoreCase("xml")
          && !tipoSaida.equalsIgnoreCase("json")))
      {
        mensagensErros.add("Favor, verificar o parâmetro 'tipoSaida'!");
      }
      if (codigoHash == null)
      {
        mensagensErros.add("Há necessidade de autorização de serviço por meio código hash!");
      }
      
      erroWs.setMensagem(mensagensErros);

      if(tipoSaida == null || (!tipoSaida.equalsIgnoreCase("xml")
          && !tipoSaida.equalsIgnoreCase("json")))
      {
        result.use(Results.xml()).from(erroWs).recursive().serialize();
      }else if (tipoSaida.equalsIgnoreCase("json")) {
        result.use(Results.json()).withoutRoot().from(erroWs).recursive().serialize();
      } else if (tipoSaida.equalsIgnoreCase("xml")) {
        result.use(Results.xml()).from(erroWs).recursive().serialize();
      }
    } else {
      if (usuarioAutorizado) {
        // tipoSaida deve ser "json" ou "xml";
        try {

          Validate validate = new Validate(this.validator);

          if (validate.url(url)) {

            if (url.startsWith("www")) {
              url = "http://" + url;
            }


            WebChecker pagina = WebChecker.from(url).withGetRequest().execute();

            int[] errorsWarningsCss = avaliacaoBusiness.getErrorCount(true, url);
            int[] errorsWarningsHtml = avaliacaoBusiness.getErrorCount(false, url);

            Checker checker = from(pagina.getContent(), url);

            checker.with(marking());
            checker.with(content());
            checker.with(presentation());
            checker.with(multimedia());
            checker.with(form());
            checker.with(behavior());

            Pattern pp = Pattern.compile("(http://www.)?([a-z]*)(.)?");
            Matcher mm = pp.matcher(url.toLowerCase().replace("https", "http"));

            if (mm.find()) {
              this.tituloSite = "";
            }

            ContentEvaluation conteudo = new ContentEvaluation(checker.getDocument());

            this.tituloSite = conteudo.retornarTituloSiteAvaliado();

            Nota nota = null;

            List<Entry<OccurrenceKey, List<Occurrence>>> mapaListaOcorrencias =
                avaliacaoBusiness.retornarCriteriosTeste(checker.check());

            nota =
                avaliacaoBusiness.obterNotaEselo(checker.getDocument(), url, mapaListaOcorrencias, false);

            this.sumarizarResultasNoResponse(checker.checkSumarized(), result);

            this.detalheAvaliacao.inicializar(avaliacaoBusiness.retornarCriterios(checker.check()),
                errorsWarningsCss, errorsWarningsHtml);



            List<SummarizedOccurrence> checkSumarized = checker.checkSumarized();

            avisosFerramentasHtmlCssW3c = new ArrayList<String>();
            List<ResumoAvaliacao> resumoErrosAvisos = obterResumoAvaliacao();

            // pegar os dados e montar o xml
       
            OccurrenceKey rn = null;
            boolean type = false;
         
            int totalErros = 0;
            int totalAvisos = 0;


            PaginaAvaliada paginaAvaliada = new PaginaAvaliada();

            paginaAvaliada.setPagina(url);
            paginaAvaliada.setTitulo(this.tituloSite);
            paginaAvaliada.setTamanho(pagina.getContentLength());
            paginaAvaliada.setDataHora(this.dataHoraAvaliacao);

            avaliacaoAutomatica.setPaginaAvaliada(paginaAvaliada);

            ResumoAvaliacaoWs resumoAvaliacaoWs = new ResumoAvaliacaoWs();

            resumoAvaliacaoWs.setNotaAvaliacao(nota.getValor());


            List<Secao> lsSecao = new ArrayList<Secao>();
            Secao secao;

            for (ResumoAvaliacao resumoAvaliacao : resumoErrosAvisos) {

              secao = new Secao();

              secao.setNome(resumoAvaliacao.getTipo().getDescription());
              secao.setQuantidadeErro(resumoAvaliacao.getQuantidadeErros());
              secao.setQuantidadeAviso(resumoAvaliacao.getQuantidadeAvisos());
              lsSecao.add(secao);

              totalErros += resumoAvaliacao.getQuantidadeErros();
              totalAvisos += resumoAvaliacao.getQuantidadeAvisos();
            }

            resumoAvaliacaoWs.setSecao(lsSecao);
            resumoAvaliacaoWs.setTotalErros(totalErros);
            resumoAvaliacaoWs.setTotalAvisos(totalAvisos);

            avaliacaoAutomatica.setResumoAvaliacao(resumoAvaliacaoWs);


            String recomendacaoCompara = "";

            List<RecomendacaoAvaliacao> lsRecomendacaoAvaliacao =
                new ArrayList<RecomendacaoAvaliacao>();

            RecomendacaoAvaliacao recomendacaoAvaliacao = new RecomendacaoAvaliacao();

            for (SummarizedOccurrence avaliacao : checkSumarized) {
              rn = OccurrenceKey.valueOf(avaliacao.getCheckPoint());
              type = avaliacao.isError();

              if (!recomendacaoCompara.equalsIgnoreCase("")) {
                lsRecomendacaoAvaliacao.add(recomendacaoAvaliacao);
                recomendacaoAvaliacao = new RecomendacaoAvaliacao();              
              }


              recomendacaoAvaliacao.setNumeroRecomendacao(avaliacao.getCheckPoint());
              recomendacaoAvaliacao.setErroRecomendacao(type);
              recomendacaoAvaliacao.setDescricaoRecomendacao(avaliacao.getDescription());
              recomendacaoAvaliacao.setNumeroRecomendacao(avaliacao.getCheckPoint());
              recomendacaoAvaliacao.setLinhasCodigoFonteRecomendacao(avaliacao.getStringLines());
              recomendacaoAvaliacao.setQuantidadeRecomendacao(avaliacao.getNumberOfOccurrences());
        

              List<CriterioWs> lsCriterioWs = new ArrayList<CriterioWs>();

              // Critérios iniciam aqui

              lsCriterioWs = criarRelatorioCriterios(rn, type);

              recomendacaoAvaliacao.setLsCriterio(lsCriterioWs);


              // Critérios terminam aqui


              recomendacaoCompara = avaliacao.getCheckPoint().toString();
            }
            lsRecomendacaoAvaliacao.add(recomendacaoAvaliacao);
            avaliacaoAutomatica.setLsRecomendacaoAvaliacao(lsRecomendacaoAvaliacao);

           if (tipoSaida.equalsIgnoreCase("json")) {
              result.use(Results.json()).withoutRoot().from(avaliacaoAutomatica).recursive()
                  .serialize();
            } else if (tipoSaida.equalsIgnoreCase("xml")) {
              result.use(Results.xml()).from(avaliacaoAutomatica).recursive().serialize();
            }

          } else {
      
            this.validator = validate.getMessage();            
            erroWs.setUrl(url);
            erroWs.setDataHora(this.dataHoraAvaliacao);

            List<String> mensagensErros = new ArrayList<String>();
            Integer contador = 0;
            for (Message mensagem : validate.getMessage().getErrors()) {
              mensagensErros.add(validate.getMessage().getErrors().get(contador).getMessage());
              contador++;
            }

            erroWs.setMensagem(mensagensErros);

            if (tipoSaida.equalsIgnoreCase("json")) {
              result.use(Results.json()).withoutRoot().from(erroWs).recursive().serialize();
            } else if (tipoSaida.equalsIgnoreCase("xml")) {
              result.use(Results.xml()).from(erroWs).recursive().serialize();
            }
    
          }


        } catch (Exception e) {
       
          e.printStackTrace();

          erroWs.setUrl(url);
          erroWs.setDataHora(this.dataHoraAvaliacao);

          List<String> mensagensErros = new ArrayList<String>();

          mensagensErros.add(e.getMessage());

          erroWs.setMensagem(mensagensErros);


          if (tipoSaida.equalsIgnoreCase("json")) {
            result.use(Results.json()).withoutRoot().from(erroWs).recursive().serialize();
          } else if (tipoSaida.equalsIgnoreCase("xml")) {
            result.use(Results.xml()).from(erroWs).recursive().serialize();
          }
        }

      } else {

        erroWs.setUrl(url);
        erroWs.setDataHora(this.dataHoraAvaliacao);
        List<String> mensagensErros = new ArrayList<String>();

        mensagensErros.add("Código hash não confere!");

        erroWs.setMensagem(mensagensErros);

        if (tipoSaida.equalsIgnoreCase("json")) {
          result.use(Results.json()).withoutRoot().from(erroWs).recursive().serialize();
        } else if (tipoSaida.equalsIgnoreCase("xml")) {
          result.use(Results.xml()).from(erroWs).recursive().serialize();
        }
       
      }
    }
  }

  public List<CriterioWs> criarRelatorioCriterios(OccurrenceKey rn, boolean isError) {

    boolean recomendacao1_1_1Adicionada = false;
    boolean recomendacao1_1_2Adicionada = false;
    boolean recomendacao1_1_7Adicionada = false;
    boolean recomendacao1_1_8Adicionada = false;

    String notExibCrit = this.utilitiesProperties.get("notExibCrit");
    
    String[] listaNotExibCrit = notExibCrit.split(" ");
    
   
    
    String codigoFonte = "";
    boolean haCodigoFonte;
    String criterioCompara = "";
    String numeroCriterio = "";
    List<CriterioWs> lsCriterioWs = new ArrayList<CriterioWs>();
    CriterioWs criterioWs;

    List<CodigoFonteCriterio> lsCodigoFonteCriterio = new ArrayList<CodigoFonteCriterio>();
    CodigoFonteCriterio codigoFonteCriterio = new CodigoFonteCriterio();

    for (Occurrence occurrence : this.detalheAvaliacao.get(rn, isError).getOcorrencias()) {
      criterioWs = new CriterioWs();
      haCodigoFonte = true;
      
      numeroCriterio = occurrence.getCriterio();
      
    String recomend =  rn.getCode().toString() + "." + numeroCriterio;
    
      for (String naotExibCrit : listaNotExibCrit ) {
        if(naotExibCrit.equalsIgnoreCase(recomend))
        {
          codigoFonte = "Não há código fonte a ser exibido.";
          haCodigoFonte = false;
          break;
        }
      }
   
      if (!rn.getCode().toString().equalsIgnoreCase("1.1") || !isError) {
        
        if (rn.getCode().toString().equalsIgnoreCase("1.1") && numeroCriterio.equalsIgnoreCase("7")
            && !recomendacao1_1_7Adicionada) {

          codigoFonte = haCodigoFonte == false ? codigoFonte : "Serviço de validação de HTML do W3C";
          recomendacao1_1_7Adicionada = true;
        } else if (rn.getCode().toString().equalsIgnoreCase("1.1")
            && numeroCriterio.equalsIgnoreCase("8") && !recomendacao1_1_8Adicionada) {
          codigoFonte = haCodigoFonte == false ? codigoFonte :  "Serviço de validação de CSS do W3C";
          recomendacao1_1_8Adicionada = true;
        } else {
          codigoFonte = haCodigoFonte == false ? codigoFonte :
              occurrence.getTag().replaceAll("&lt;", "<").replaceAll("&gt;", ">")
                  .replaceAll("&nbsp", " ");
        }

        if (!criterioCompara.equalsIgnoreCase(occurrence.getCriterio().toString())) {

          if (!criterioCompara.equalsIgnoreCase("")) {
            criterioWs.setCodigoFonteCriterio(lsCodigoFonteCriterio);
            codigoFonteCriterio = new CodigoFonteCriterio();
          }

          criterioWs.setNumeroCriterio(numeroCriterio);
          criterioWs.setNomeCriterio(criterioProperties.getDescricao(rn.getCode() + "."
              + numeroCriterio));
          criterioWs.setErroCriterio(occurrence.isError());
          criterioWs.setQuantidadeCriterio(verificaQuantidadeCriterio(rn, isError, numeroCriterio));


          codigoFonteCriterio.setNumeroLinha(haCodigoFonte == false ? 0 :occurrence.getLine());
          codigoFonteCriterio.setConteudoLinha(haCodigoFonte == false ? codigoFonte : "<![CDATA[" + codigoFonte + "]]>");
          lsCodigoFonteCriterio.add(codigoFonteCriterio);


          criterioCompara = occurrence.getCriterio();
        } else {
          codigoFonteCriterio = new CodigoFonteCriterio();
          codigoFonteCriterio.setNumeroLinha(haCodigoFonte == false ? 0 :occurrence.getLine());
          codigoFonteCriterio.setConteudoLinha(haCodigoFonte == false ? codigoFonte :"<![CDATA["
              + occurrence.getTag().replaceAll("&lt;", "<").replaceAll("&gt;", ">")
                  .replaceAll("&nbsp", " ") + "]]>");
          lsCodigoFonteCriterio.add(codigoFonteCriterio);

               }

      } else {
        if (numeroCriterio.equalsIgnoreCase("1") && !recomendacao1_1_1Adicionada) {

          if (!criterioCompara.equalsIgnoreCase("")) {
            criterioWs.setCodigoFonteCriterio(lsCodigoFonteCriterio);
            codigoFonteCriterio = new CodigoFonteCriterio();
          }

          criterioWs.setNumeroCriterio(numeroCriterio);
          criterioWs.setNomeCriterio(criterioProperties.getDescricao(rn.getCode() + "."
              + numeroCriterio));
          criterioWs.setErroCriterio(occurrence.isError());
          criterioWs.setQuantidadeCriterio(verificaQuantidadeCriterio(rn, isError, numeroCriterio));


          // Seta-se "0" por não possuir essa informação
          codigoFonteCriterio.setNumeroLinha(0);
          codigoFonteCriterio.setConteudoLinha("Serviço de validação de HTML do W3C");
          lsCodigoFonteCriterio.add(codigoFonteCriterio);

    
          criterioCompara = occurrence.getCriterio();
          recomendacao1_1_1Adicionada = true;
        }

        if (numeroCriterio.equalsIgnoreCase("2") && !recomendacao1_1_2Adicionada) {
          if (!criterioCompara.equalsIgnoreCase("")) {
            criterioWs.setCodigoFonteCriterio(lsCodigoFonteCriterio);
            codigoFonteCriterio = new CodigoFonteCriterio();          
          }

          criterioWs.setNumeroCriterio(numeroCriterio);
          criterioWs.setNomeCriterio(criterioProperties.getDescricao(rn.getCode() + "."
              + numeroCriterio));
          criterioWs.setErroCriterio(occurrence.isError());
          criterioWs.setQuantidadeCriterio(verificaQuantidadeCriterio(rn, isError, numeroCriterio));


          // Seta-se "0" por não possuir essa informação
          codigoFonteCriterio.setNumeroLinha(0);
          codigoFonteCriterio.setConteudoLinha("Serviço de validação de CSS do W3C");
          lsCodigoFonteCriterio.add(codigoFonteCriterio);

          criterioCompara = occurrence.getCriterio();
          recomendacao1_1_2Adicionada = true;
        }

      }
      if (lsCodigoFonteCriterio.size() > 0) {
        criterioWs.setCodigoFonteCriterio(lsCodigoFonteCriterio);
        lsCodigoFonteCriterio = new ArrayList<CodigoFonteCriterio>();
        lsCriterioWs.add(criterioWs);
      }   
    }   
    return lsCriterioWs;


  }

  private Integer verificaQuantidadeCriterio(OccurrenceKey rn, boolean isError,
      String numeroCriterio) {
    Integer quantidadeCriterios = 0;

    for (Occurrence occurrence2 : this.detalheAvaliacao.get(rn, isError).getOcorrencias()) {
      if (occurrence2.getCriterio().toString().equalsIgnoreCase(numeroCriterio)) {
        quantidadeCriterios++;
      }
    }

    return quantidadeCriterios;

  }

}
