package br.com.ases.model.utilities;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.HeadMethod;
import org.apache.commons.httpclient.params.HttpConnectionParams;
import org.apache.commons.httpclient.params.HttpMethodParams;

import br.com.ases.model.entity.Contato;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.interceptor.multipart.UploadedFile;
import br.com.caelum.vraptor.validator.ValidationMessage;

public class Validate {

  private final Validator validator;

  public Validate(Validator validator) {
    this.validator = validator;
  }

  public boolean contato(Contato contato) {
    boolean isValido = true;

    if (contato.getNome() == null) {
      this.validator.add(new ValidationMessage("Favor preencher o campo NOME!", "error"));
      isValido = false;
    } else if (contato.getNome().length() > 200) {
      this.validator.add(new ValidationMessage("O campo NOME aceita até 200 caracteres.",
          "error"));
      isValido = false;
    }

    if (contato.getEmail() == null) {
      this.validator.add(new ValidationMessage("Favor preencher o campo E-MAIL!", "error"));
      isValido = false;
    } else {
      Pattern p = Pattern.compile("^[\\w-]+(\\.[\\w-]+)*@([\\w-]+\\.)+[a-zA-Z]{2,7}$");
      Matcher m = p.matcher(contato.getEmail());
      if (!m.find()) {
        // this.validator.add(new
        // ValidationMessage("N&atilde;o foi poss&iacute;vel enviar a mensagem!  E-MAIL '"+contato.getEmail()+"' considerado inv&aacute;lido!",
        // "error"));
        this.validator.add(new ValidationMessage("E-MAIL considerado inválido!", "error"));
        isValido = false;
      }
    }

    if (contato.getAssunto() == null) {
      this.validator.add(new ValidationMessage(
          "Favor escolher uma das opções no campo ASSUNTO!", "error"));
      isValido = false;
    }

    if (contato.getMensagem() == null) {
      this.validator.add(new ValidationMessage("Favor preencher o campo MENSAGEM!", "error"));
      isValido = false;
    } else if (contato.getMensagem().length() > 500) {
      this.validator.add(new ValidationMessage(
          "O campo MENSAGEM aceita at&eacute; 500 caracteres.", "error"));
      isValido = false;
    }

    return isValido;
  }



  public boolean url(String campo) {

    boolean isValido = true;
    int codResponse;
    
   // HttpMethod metodoRequisicaoGET = null;
    //HttpClient clienteHTTPJakartaCommons;
    //URL UrlConvertida;

    if (campo == null || campo.length() <= 10) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! Favor preencher o campo URL.",
              "warning"));
      isValido = false;
    } else {

    	  codResponse = verificarConexao(campo, "http.agent", "Jakarta Commons-HttpClient/3.1");
          if (codResponse != 200) {
            codResponse = verificarConexao(campo, "user-agent", "NewUseAgent/1.0");
            if (codResponse == -1) {
            	 this.validator.add(new ValidationMessage(
                         "Não foi possível realizar a avaliação! URL " + campo
                             + " é considerada inválida.", "warning"));
            	isValido = false;
            }
          }
          

    }

    return isValido;
  }
  
  

  private Integer verificarConexao(String campo, String usuario, String agente) {

    int codResponse = -1;
    System.setProperty("jsse.enableSNIExtension", "false");
    HttpMethod metodoRequisicaoGET = null;
    HttpClient clienteHTTPJakartaCommons;
    URL UrlConvertida;

    try {
    	
    	 clienteHTTPJakartaCommons = new HttpClient();
         clienteHTTPJakartaCommons.getParams().setParameter(HttpMethodParams.RETRY_HANDLER,
             new DefaultHttpMethodRetryHandler(3, false));
         clienteHTTPJakartaCommons.getParams().setParameter("http.protocol.allow-circular-redirects",
             true);
         clienteHTTPJakartaCommons.getParams().setParameter(HttpConnectionParams.CONNECTION_TIMEOUT, new Integer(10000));

        
         metodoRequisicaoGET = new HeadMethod(campo);// GetMethod(UrlConvertida.toExternalForm());// URLEncoder.encode(UrlConvertida.toExternalForm(),
         // "UTF-8"));

         metodoRequisicaoGET.setRequestHeader(usuario, agente);
         // metodoRequisicaoGET.setFollowRedirects(true);
        
         codResponse = clienteHTTPJakartaCommons.executeMethod(metodoRequisicaoGET);
         
        // if(codResponse >= 400 && codResponse < 500)
        	 if(codResponse != 200)
         {    	  
          metodoRequisicaoGET = new GetMethod(campo);
       	  codResponse = clienteHTTPJakartaCommons.executeMethod(metodoRequisicaoGET);
         }

      //UrlConvertida = new URL(campo);

      /*clienteHTTPJakartaCommons = new HttpClient();
      clienteHTTPJakartaCommons.getParams().setParameter(HttpMethodParams.RETRY_HANDLER,
          new DefaultHttpMethodRetryHandler(5, false));
      clienteHTTPJakartaCommons.getParams().setParameter("http.protocol.allow-circular-redirects",
          true);

      metodoRequisicaoGET = new GetMethod(campo);//UrlConvertida.toExternalForm());

      metodoRequisicaoGET.setRequestHeader(usuario, agente);
      metodoRequisicaoGET.setFollowRedirects(true);

      codResponse = clienteHTTPJakartaCommons.executeMethod(metodoRequisicaoGET);*/

      

    } catch (MalformedURLException e) {
      e.printStackTrace();
      // this.validator.add(new ValidationMessage(
      // "N&atilde;o foi poss&iacute;vel realizar a avalia&ccedil;&atilde;o! URL " + campo
      // + " é considerada inválida.", "warning"));
      return -1;
    } catch (IOException e) {
      e.printStackTrace();
      // this.validator.add(new ValidationMessage(
      // "N&atilde;o foi poss&iacute;vel realizar a avalia&ccedil;&atilde;o! URL " + campo
      // + " é considerada inválida.", "warning"));
      return -1;
    }
    finally
    {
    	if (metodoRequisicaoGET != null) {
    	metodoRequisicaoGET.releaseConnection();
    	}    	
    }
    return codResponse;

  }

  public boolean condigoFonte(String campo) {

    if (campo == null || campo.length() <= 10) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! Favor preencher o campo Código a analisar.",
              "warning"));
      return false;
    }

    String reg = "<html.*?>(.*)<\\/html>";

    Pattern p = Pattern.compile(reg, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
    Matcher m = p.matcher(campo);

    if (!m.find()) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! O código fonte não é do tipo HTML ou XHTML.",
              "warning"));
      return false;
    }

    if (campo.length() > 1048576) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! Tamanho máximo permitido para código fonte é até 1024KB.",
              "warning"));
      return false;
    }

    return true;

  }

  public boolean uploadForm(UploadedFile file) {

    if (file == null) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! Favor realizar o upload do arquivo.",
              "warning"));
      return false;
    }

    String fileType = file.getContentType();
    if (!(fileType.equals("text/html") || fileType.equals("application/xhtml+xml")
        || fileType.equals("application/xhtml") || fileType.equals("application/xml"))) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! As extensções permitidas para o arquivo são: .xht, .htm, .html ou .xhtml.",
              "warning"));
      return false;
    }

    if (file.getSize() > 1048576) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar o upload do arquivo! Tamanho máximo permitido é de 1024KB.",
              "warning"));
      return false;
    }

    return true;

  }

  public boolean conteudoUploadForm(String html) {

    String reg = "<html.*?>(.*)<\\/html>";

    Pattern p = Pattern.compile(reg, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
    Matcher m = p.matcher(html);

    if (!m.find()) {
      this.validator
          .add(new ValidationMessage(
              "Não foi possível realizar a avaliação! O conteúdo do arquivo não é do tipo HTML ou XHTML.",
              "warning"));
      return false;
    }

    return true;
  }

  public Validator getMessage() {
    return this.validator;
  }

}
